@@ -5,13 +5,13 @@ class Redis
     @@timeout = 5
     attr_reader :ring
 
-    def initialize(addresses, options = { })
-      nodes = addresses.map do |address|
-        ::Redis::Store.new _merge_options(address, options)
-      end
-
+    def initialize(addresses, options = {})
       _extend_namespace options
-      @ring = Redis::HashRing.new nodes
+      @ring = options[:ring] || Redis::HashRing.new([], options[:replicas] || Redis::HashRing::POINTS_PER_SERVER)
+
+      addresses.each do |address|
+        @ring.add_node(::Redis::Store.new _merge_options(address, options))
+      end
     end
 
     def nodes
@@ -19,7 +19,7 @@ class Redis
     end
 
     def reconnect
-      nodes.each {|node| node.reconnect }
+      nodes.each { |node| node.reconnect }
     end
 
     def set(key, value, options = nil)
@@ -57,10 +57,10 @@ class Redis
       end
 
       def _merge_options(address, options)
-        address.merge({
-          :timeout => options[:timeout] || @@timeout, 
+        address.merge(
+          :timeout => options[:timeout] || @@timeout,
           :namespace => options[:namespace]
-        })
+        )
       end
   end
 end
