@@ -41,6 +41,7 @@ class MockRedis
       block.call
     end
   end
+  alias_method :pipelined, :multi
 
   def expire(*a)
     @expires << a
@@ -49,7 +50,6 @@ class MockRedis
   def has_expire?(*a)
     @expires.include?(a)
   end
-
 end
 
 class MockTtlStore < MockRedis
@@ -79,7 +79,7 @@ describe MockTtlStore do
 
     describe 'with nx and ex option' do
       it 'must call super with key and value and options' do
-        set_options = {nx: true, ex: 3600}
+        set_options = { nx: true, ex: 3600 }
         redis.set(key, mock_value, set_options)
         redis.has_set?(key, mock_value, set_options).must_equal true
       end
@@ -94,13 +94,17 @@ describe MockTtlStore do
       end
 
       it 'must not call expire' do
-        MockTtlStore.any_instance.expects(:expire).never
-
+        redis.expects(:expire).never
         redis.setnx(key, mock_value)
       end
     end
 
     describe 'with expiry' do
+      it 'uses the mutli command to chain commands' do
+        redis.expects(:multi)
+        redis.setnx(key, mock_value, options)
+      end
+
       it 'must call setnx with key and value and set raw to true' do
         redis.setnx(key, mock_value, options)
         redis.has_setnx?(key, mock_value, :raw => true).must_equal true
@@ -110,6 +114,44 @@ describe MockTtlStore do
         redis.setnx(key, mock_value, options)
         redis.has_expire?(key, options[:expire_after]).must_equal true
       end
+
+      describe 'avoiding multi commands' do
+        let(:options) { { :expire_after => 3600, :avoid_multi_commands => true } }
+
+        it 'uses the redis pipelined feature to chain commands' do
+          redis.expects(:pipelined)
+          redis.setnx(key, mock_value, options)
+        end
+
+        it 'must call setnx with key and value and set raw to true' do
+          redis.setnx(key, mock_value, options)
+          redis.has_setnx?(key, mock_value, :raw => true).must_equal true
+        end
+
+        it 'must call expire' do
+          redis.setnx(key, mock_value, options)
+          redis.has_expire?(key, options[:expire_after]).must_equal true
+        end
+      end
+
+      describe 'using a redis cluster' do
+        let(:options) { { :expire_after => 3600, :cluster => %w[redis://127.0.0.1:6379/0] } }
+
+        it 'uses the redis pipelined feature to chain commands' do
+          redis.expects(:pipelined)
+          redis.setnx(key, mock_value, options)
+        end
+
+        it 'must call setnx with key and value and set raw to true' do
+          redis.setnx(key, mock_value, options)
+          redis.has_setnx?(key, mock_value, :raw => true).must_equal true
+        end
+
+        it 'must call expire' do
+          redis.setnx(key, mock_value, options)
+          redis.has_expire?(key, options[:expire_after]).must_equal true
+        end
+      end
     end
   end
 end
